# Documentation

Ce module permet de gérer des listings d'items.

Un item peut etre : 

* un produit
* une page CMS
* une categorie
* ...


## Utiliser un listing

### Les services

Le module propose 2 services à utiliser :

* Listing : qui permet de gerer les listing de manieres général
* ListingProduct : qui permet de gérer des listings de produits


Pour gérer la récupération des données, il faut injecter au service listing le model que l'on souhaite utiliser pour récupérer les données.

Ce model doit implement la classe `ItemLoaderInterface` et les méthodes necessaires de l'interface.

### Les critères

Pour filtrer les résultats du listing, on peut ajouter des criteres qui vont permettre de réduire le nombre de résultats renvoyé.

Par exemple, pour un listing de produit pour la catégorie 1 :
    
    <?php
    
    $listing = new ListingProduct(new Produit());
    $listing->addCriteria('categorie', new \Bydedi\Module\Listing\Criteria(1));
    
### Service Listing

Exemple d'utilisation :

    <?php
    
    // la classe ItemLoader n'existe passe
    // instance d'un objet ItemLoader classe qui contient la logique pour récupérer les données en bdd
    $itemLoader = new ItemLoader();
    
    // instance du service Listing
    $listing = new \Bydedi\Module\Listing\Service\Listing($itemLoader);
    
    // ajout d'un critere pour filtrer le listing
    $listing->addCriteria('criteria1', new \Bydedi\Module\Listing\Criteria('value'));
    
    // récupération des items à afficher
    $items = $listing->getItemsToDisplay();

### Service ListingProduit
    

## Créer un service Listing

### Construire un listing

Pour construire un nouveau service Listing, il faut créer une nouvelle classe dans le dossier `/Service`

Cette classe doit herité de `ListingAbstract` qui permet de déclarer les méthodes de base utilisé pour un listing



## Les interfaces

Liste des interfaces et leur utilité :

* ItemLoaderInterface : interface à implémenter sur les objets Model qui contiennent la logique pour récupérer les données depuis la base de données
* CriteriaInterface : interface pour un objet qui peut etre un critere pour filtrer le listing
* ListingInterface : interface pour un objets de service Listing


## Test unitaire

Chaque classe de service doit etre accompagné de son tests unitaire dans le dossier `Test/Service`

Et nommé avec le nom de la classe puis le suffixe `Test`