<?php

namespace Bydedi\Module\Listing\Service;

use Bydedi\Module\Listing\Criteria;
use Bydedi\Module\Listing\CriteriaInterface;
use Bydedi\Module\Listing\ListingAbstract;
use Caracteristique;
use CaracteristiqueValeur;
use Categorie;
use Texte;
use Tools;

/**
 * Class ListingProduct
 * @package Bydedi\Module\Listing\Service
 */
class ListingProduct extends ListingAbstract
{
    /**
     * @var array
     */
    protected $caraProducts;
    /**
     * @var string
     */
    protected $search;
    /**
     * @var array
     */
    protected $prix;

    /**
     * @var int
     */
    protected $prixmax;
    /**
     * @var int
     */
    protected $prixmin;

    /**
     * @var string
     */
    protected $caracteristiqueUrl;
    /**
     * @var array
     */
    protected $statut;

    /**
     * @return $this|mixed
     */
    public function reset()
    {
        parent::reset();

        $this->limit = 16;

        $this->caraProducts = array();
        $this->search = null;
        $this->prix = array();
        $this->statut = array();

        $this->criteria['caracteristique'] = array();

        $this->prixmin = 0;
        $this->prixmin = 0;

        return $this;
    }

    /**
     * @param int $statut
     * * @return $this
     */
    public function addStatut($statut)
    {
        $this->statut[] = $statut;
        return $this;
    }

    /**
     * @param $categorie
     * @return $this
     */
    public function addCategorie(CriteriaInterface $categorie)
    {
        return $this->addCriteria('categorie', $categorie);
    }

    /**
     * @param $gamme
     * @return $this
     */
    public function addGamme(CriteriaInterface $gamme)
    {
        return $this->addCriteria('gamme', $gamme);
    }

    /**
     * @param $marque
     * @return $this
     */
    public function addMarque(CriteriaInterface $marque)
    {
        return $this->addCriteria('marque', $marque);
    }

    /**
     * @param $caracteristique
     * @return $this
     */
    public function addCaracteristique(CriteriaInterface $caracteristique)
    {
        return $this->addCriteria('caracteristique', $caracteristique);
    }

    /**
     * @param $search string
     * @return $this
     */
    public function setSearch($search)
    {
        $this->setCriteria('search', new Criteria($search));
    }


    /**
     * @param $prix
     */
    public function setPrix($prix)
    {
        $this->setCriteria('prix', new Criteria($prix));
    }

    /**
     * @return int
     */
    public function getPrixMax()
    {
        return $this->prixmax;
    }

    /**
     * @return int
     */
    public function getPrixMin()
    {
        return $this->prixmin;
    }

    /**
     * @return array
     */
    public function getCaracteristiqueSelect()
    {
        $arrCara = array();
        if (!empty($this->criteria['caracteristique'])) {
            $caracteristique = new Caracteristique();

            $url = (!empty($this->caracteristiqueUrl)) ?
                str_replace('/' . $this->caracteristiqueUrl, '', $this->request->getRequestUri()) :
                $this->request->getRequestUri();

            foreach ($this->criteria['caracteristique'] as $cara) {
                $caraTmp = array($cara->toArray());
                $caracteristique->setUrl($caraTmp, $this->criteria['caracteristique'], $url);
                $arrCara[] = $caraTmp[0];
            }
        }

        return $arrCara;
    }

    /**
     * function to get all cara match with the product in the listing
     *
     * @param null $caracteristiqueId
     * @return array
     */
    public function getCaracteristique($caracteristiqueId = null)
    {
        if (empty($this->caraProducts)) {
            $this->loadCaracteristique();
        }

        if (!is_null($caracteristiqueId)) {
            if (!is_array($caracteristiqueId)) {
                $caracteristiqueId = array($caracteristiqueId);
            }

            $caracs = array_map(
                function ($value) use ($caracteristiqueId) {
                    if (in_array($value['caracteristique_id'], $caracteristiqueId)) {
                        return $value;
                    }
                    return null;
                },
                $this->caraProducts
            );
            return array_filter($caracs);
        }

        return $this->caraProducts;
    }

    /**
     * @param $url
     */
    public function setCaracteristiqueUrl($url)
    {
        $this->caracteristiqueUrl = $url;

        $caracteristique = new Caracteristique();

        $arrCara = $caracteristique->parseUrl($this->caracteristiqueUrl);

        foreach ($arrCara as $caraId) {
            $this->addCaracteristique(new CaracteristiqueValeur($caraId));
        }
    }

    /**
     * function to load all products match with the criteria add : Categorie / Gamme / Marque / etc...
     *
     * @param array $param
     *
     */
    public function loadItems($param = array())
    {
        $this->setCriteria('noBloc', new Criteria(true));

        if (is_null($this->search) == false) {
            if (is_array($this->search)) {
                list($idx, $search) = $this->search;
            } else {
                $idx = 'motclef';
                $search = $this->search;
            }
            $param[$idx] = $search;
        }

        /*if (!empty($this->statut)) {
            $param['statut'] = $this->statut;
        }*/

        $param = $this->prepareCriteria();

        if (!empty($param['categorie'])) {
            $param['categorie_ext_id'] = $param['categorie'][0];
            unset($param['categorie']);
        }

        $this->items = $this->itemLoader->getSearch($param);

        $this
            ->setPrixMax()
            ->setPrixMin()
        ;
    }

    /**
     * @param null $items
     * @return $this
     */
    public function setPrixMax($items = null)
    {
        if (is_null($items)) {
            $items = $this->items;
        }

        foreach ($items as $item) {
            if ($item['produit_prix_ttc'] > $this->prixmax) {
                $this->prixmax = $item['produit_prix_ttc'];
            }
        }

        return $this;
    }

    /**
     * @param null $items
     * @return $this
     */
    public function setPrixMin($items = null)
    {
        if (is_null($items)) {
            $items = $this->items;
        }

        foreach ($items as $item) {
            if ($item['produit_prix_ttc'] < $this->prixmin) {
                $this->prixmin = $item['produit_prix_ttc'];
            }
        }

        return $this;
    }

    /**
     *
     * function to load the list of the cara for the products
     */
    protected function loadCaracteristique()
    {
        $caracteristique = new Caracteristique();

        $arrValeur = $caracteristique->getValeurProduit([
            'listProduit' => Tools::extractValue('produit_id', $this->items)
        ]);

        if (!empty($arrValeur)) {
            $url = (!empty($this->caracteristiqueUrl)) ?
                str_replace('/' . $this->caracteristiqueUrl, '', $this->request->getRequestUri()) :
                $this->request->getRequestUri();

            $caracteristique->setUrl($arrValeur, $this->criteria['caracteristique'], $url);
        }

        $this->caraProducts = $caracteristique->constructCara(
            $arrValeur,
            $this->criteria['caracteristique'],
            $this->items
        );
    }

    /**
     * @return array
     */
    public function getReferencement()
    {
        $texte = new Texte();

        $ref = array(
            'title' => '',
            'description' => '',
            'keywords' => '',
            'other' => ''
        );

        list($type, $obj, $suffixeTxt) = $this->resolveTypeListing();

        if (!empty($obj)) {
            $ref['title'] = (!empty($obj->{$type . '_nom_meta_title'})) ?
                $obj->{$type . '_nom_meta_title'} : $texte->getOne('ref_title_' . $suffixeTxt, true, $obj->toArray());
            $ref['description'] = (!empty($obj->{$type . '_nom_meta_desc'})) ?
                $obj->{$type . '_nom_meta_desc'} : $texte->getOne('ref_desc_' . $suffixeTxt, true, $obj->toArray());
            $ref['keywords'] = (!empty($obj->{$type . '_nom_meta_kw'})) ?
                $obj->{$type . '_nom_meta_kw'} : $texte->getOne('ref_kw_' . $suffixeTxt, true, $obj->toArray());
            $ref['other'] = (!empty($obj->{$type . '_nom_meta_other'})) ?
                $obj->{$type . '_nom_meta_other'} : $texte->getOne('ref_other_' . $suffixeTxt, true, $obj->toArray());
        } else {
            $ref['title'] = $texte->getOne('ref_title_' . $suffixeTxt, true, array('motclef' => $this->search));
            $ref['description'] = $texte->getOne('ref_desc_' . $suffixeTxt, true, array('motclef' => $this->search));
        }

        return $ref;
    }

    /**
     * @return array
     */
    protected function resolveTypeListing()
    {
        $type = null;
        $obj = null;
        $suffixeTxt = null;

        if (!empty($this->criteria['categorie'])) {
            $type = 'categorie';
            $suffixeTxt = (empty($this->criteria['categorie'][0]->parent_id)) ? 'parent' : 'categorie';
            $obj = $this->criteria['categorie'][0];
            if (!empty($this->criteria['categorie'][0]->parent_id)) {
                $parent = new Categorie($this->criteria['categorie'][0]->parent_id);
                $obj->parent_nom_nom = $parent->categorie_nom_nom;
            }
        } elseif (!empty($this->criteria['marque'])) {
            $type = $suffixeTxt = 'marque';
            $obj = $this->criteria['marque'][0];
        } elseif (!empty($this->criteria['gamme'])) {
            $type = $suffixeTxt = 'gamme';
            $obj = $this->criteria['gamme'][0];
        } elseif (!empty($this->search)) {
            $type = $suffixeTxt = 'search';
        }

        return array($type, $obj, $suffixeTxt);
    }

    /**
     * @deprecated
     * @return array
     */
    public function getProductsToDisplay()
    {
        return $this->getItemsToDisplay();
    }

    /**
     * @deprecated
     * @return int
     */
    public function getCountProducts()
    {
        return $this->getCountItems();
    }

    /**
     * @deprecated
     * @return array
     */
    public function getProducts()
    {
        return $this->getItems();
    }
}
